@@ -1,7 +1,7 @@
 package SVK::Command::Admin;
 use strict;
 use SVK::I18N;
-our $VERSION = $SVK::VERSION;
+use SVK::Version;  our $VERSION = $SVK::VERSION;
 
 use base qw( SVK::Command );
 
@@ -36,7 +36,7 @@ sub run {
     (system(
         'svnadmin',
         $command,
-        ($target ? $target->{repospath} : ()),
+        ($target ? $target->repospath : ()),
         @arg
     ) >= 0) or die loc("Could not run %1: %2", 'svnadmin', $?);
 
