@@ -4,7 +4,7 @@ use SVK::Version;  our $VERSION = $SVK::VERSION;
 
 use base qw( SVK::Command );
 use SVK::I18N;
-use SVK::Util qw( get_encoder );
+use SVK::Util qw( get_encoder can_run );
 use autouse 'File::Find' => qw(find);
 
 sub parse_arg { shift; @_ ? @_ : 'index'; }
@@ -15,7 +15,7 @@ sub parse_arg { shift; @_ ? @_ : 'index'; }
 sub run {
     my $self = shift;
 
-    if(defined($ENV{SVKPAGER})){
+    if($ENV{SVKPAGER} && can_run($ENV{SVKPAGER})){
         eval '$ENV{PAGER}=$ENV{SVKPAGER};use IO::Pager;IO::Pager->new(*STDOUT)';
     }
 
@@ -30,7 +30,7 @@ sub run {
             );
             $self->brief_usage ($_) for sort @cmd;
         }
-        elsif (my $cmd = eval { $self->get_cmd ($topic) }) {
+        elsif (my $cmd = eval { SVK::Command->get_cmd ($topic) }) {
             $cmd->usage(1);
         }
         elsif (my $file = $self->_find_topic($topic)) {
