@@ -6,9 +6,7 @@ use base qw( SVK::Command );
 use constant opt_recursive => 0;
 use SVK::XD;
 use SVK::I18N;
-use SVK::Util qw( to_native get_encoder );
-use Date::Parse qw(str2time);
-use Date::Format qw(time2str);
+use SVK::Util qw( to_native get_encoder reformat_svn_date );
 
 sub options {
     ('r|revision=s'  => 'rev',
@@ -27,9 +25,11 @@ sub run {
     my ($self, @arg) = @_;
     my $exception = '';
 
-    for (@arg) {
-        eval { _do_list($self, 0, $_); print "\n" };
-        $exception .= "$@\n" if $@;
+    while (my $arg = shift @arg) {
+	$arg = $arg->as_depotpath;
+        eval { _do_list($self, 0, $self->apply_revision($arg));
+	       print "\n" if @arg };
+        $exception .= "$@" if $@;
     }
 
     die($exception) if($exception);
@@ -38,24 +38,21 @@ sub run {
 sub _do_list {
     my ($self, $level, $target) = @_;
     my $pool = SVN::Pool->new_default;
-    $target->as_depotpath ($self->resolve_revision($target,$self->{rev}));
     my $root = $target->root;
-    unless ((my $kind = $root->check_path ($target->{path})) == $SVN::Node::dir) {
-       die loc("Path %1 is not a versioned directory\n", $target->{path})
+    unless ((my $kind = $root->check_path ($target->path_anchor)) == $SVN::Node::dir) {
+       die loc("Path %1 is not a versioned directory\n", $target->path_anchor)
            unless $kind == $SVN::Node::file;
        return;
     }
 
-    # XXX: SVK::Target should take care of this.
-    $target->{depotpath} =~ s|/$||;
-    my $entries = $root->dir_entries ($target->{path});
+    my $entries = $root->dir_entries ($target->path_anchor);
     my $enc = get_encoder;
     for (sort keys %$entries) {
 	my $isdir = ($entries->{$_}->kind == $SVN::Node::dir);
 
         if ($self->{verbose}) {
-	    my $rev = $root->node_created_rev ("$target->{path}/$_");
-            my $fs = $target->{'repos'}->fs;
+	    my $rev = $root->node_created_rev ($target->path."/$_");
+            my $fs = $target->repos->fs;
 
             my $svn_date =
                 $fs->revision_prop ($rev, 'svn:date');
@@ -66,12 +63,12 @@ sub _do_list {
 	    # Additional fields for verbose: revision author size datetime
             printf "%7ld %-8.8s %10s %12s ", $rev,
                 $fs->revision_prop ($rev, 'svn:author'),
-                ($isdir) ? "" : $root->file_length ("$target->{path}/$_"),
-		time2str ("%b %d %H:%M", str2time ($svn_date));
+                ($isdir) ? "" : $root->file_length ($target->path."/$_"),
+                reformat_svn_date("%b %d %H:%M", $svn_date);
         }
 
         if ($self->{'fullpath'}) {
-	    my $dpath = $target->{path};
+	    my $dpath = $target->path_anchor;
 	    to_native ($dpath, 'path', $enc);
 	    $dpath .= '/' unless $dpath eq '/';
             print '/'.$target->depotname.$dpath;
@@ -84,8 +81,7 @@ sub _do_list {
 
 	if ($isdir && ($self->{recursive}) &&
 	    (!$self->{'depth'} ||( $level < $self->{'depth'} ))) {
-	    _do_list($self, $level+1, $target->new (path => "$target->{path}/$_",
-						    depotpath => "$target->{depotpath}/$_"));
+	    _do_list($self, $level+1, $target->new->descend($_));
 	}
     }
 }
@@ -104,9 +100,9 @@ SVK::Command::List - List entries in a directory from depot
 
 =head1 OPTIONS
 
- -r [--revision] REV	: act on revision REV instead of the head revision
+ -r [--revision] REV    : act on revision REV instead of the head revision
  -R [--recursive]       : descend recursively
- -d [--depth] LEVEL	: recurse at most LEVEL levels deep; use with -R
+ -d [--depth] LEVEL     : recurse at most LEVEL levels deep; use with -R
  -f [--full-path]       : show pathname for each entry, instead of a tree
  -v [--verbose]         : print extra information
 
