@@ -26,10 +26,15 @@ sub _log_remote_rev {
     my ( $target, $remoteonly ) = @_;
 
     # don't bother if this repository has no mirror
-    return if !$target->mirror->entries;
+    return if !$target->mirror->_entries;
 
-    # XXX: if there's no $m why do we need to be able to lookup?
-    my $m = $target->is_mirrored || 'SVN::Mirror';
+    # we might be running log on a path containing mirrors.
+    # FIXME: resolve when log outside mirror anchor
+    my $m = $target->is_mirrored;
+    return sub { my $prop = $target->repos->fs->revision_prop($_[0], 'svm:headrev') or return;
+		 my %rev = map {split (':', $_, 2)} $prop =~ m/^.*$/mg;
+		 return (values %rev)[0];
+	     } unless $m;
     return sub {
         my $rrev = $m->find_remote_rev( $_[0], $target->repos );
         return $rrev;
@@ -62,14 +67,14 @@ sub run {
 
     my ($fromrev, $torev);
     # move to general revspec parser in svk::command
-    if ($self->{revspec}) {
+    if (defined $self->{revspec}) {
         ($fromrev, $torev) = $self->resolve_revspec($target);
 	$torev ||= $fromrev;
     }
     $target = $target->as_depotpath($self->find_base_rev($target))
 	if $target->isa('SVK::Path::Checkout');
-    $fromrev ||= $target->revision;
-    $torev ||= 0;
+    $fromrev = $target->revision unless defined $fromrev;
+    $torev = 0 unless defined $torev;
     $self->{cross} ||= 0;
 
     my $get_remoterev = _log_remote_rev($target);
@@ -79,10 +84,16 @@ sub run {
 	$fromrev = min ($target->revision, $fromrev);
 	$torev = min ($target->revision, $torev);
     }
+
+    my $select_filter = $self->{select_filter};
+    if (defined $self->{limit}) {
+        $select_filter .= ' | ' if defined $select_filter and length $select_filter;
+        $select_filter .= "head " . $self->{limit};
+    }
     require SVK::Log::FilterPipeline;
     my $pipeline = SVK::Log::FilterPipeline->new(
         presentation  => $presentation_filter,
-        selection     => $self->{select_filter},
+        selection     => $select_filter,
         output        => undef,
         indent        => 0,
         get_remoterev => $get_remoterev,
@@ -92,7 +103,6 @@ sub run {
     );
     _get_logs(
         root     => $target->root,
-        limit    => $self->{limit},
         path     => $target->path_anchor,
         fromrev  => $fromrev,
         torev    => $torev,
@@ -105,10 +115,9 @@ sub run {
 
 sub _get_logs {
     my (%args) = @_;
-    my   (   $root, $limit, $path, $fromrev, $torev, $cross, $pipeline, $cb_log) = 
-    @args{qw/ root   limit   path   fromrev   torev   cross   pipeline   cb_log/};
+    my   (   $root, $path, $fromrev, $torev, $cross, $pipeline, $cb_log) = 
+    @args{qw/ root   path   fromrev   torev   cross   pipeline   cb_log/};
 
-    $limit ||= -1;
     my $fs = $root->fs;
     my $reverse = ($fromrev < $torev);
     my @revs;
@@ -148,8 +157,6 @@ sub _get_logs {
             return 1 if $rev > $fromrev; # next
             return 0 if $rev < $torev;   # last
 
-            return 0 if !$limit--; # last
-
             if ($reverse) {
                 unshift @revs, $rev;
                 return 1;
@@ -176,8 +183,8 @@ $chg->[$SVN::Fs::PathChange::replace] = 'R';
 
 sub do_log {
     my (%arg) = @_;
-    my (    $cross, $fromrev, $limit, $path, $repos, $torev, $pipeline, $cb_log ) =
-    @arg{qw/ cross   fromrev   limit   path   repos   torev   pipeline   cb_log /};
+    my (    $cross, $fromrev, $path, $repos, $torev, $pipeline, $cb_log ) =
+    @arg{qw/ cross   fromrev   path   repos   torev   pipeline   cb_log /};
 
     $cross ||= 0;
     my $pool = SVN::Pool->new_default;
@@ -185,7 +192,6 @@ sub do_log {
     my $rev = $fromrev > $torev ? $fromrev : $torev;
     _get_logs (
         root     => $fs->revision_root($rev),
-        limit    => $limit,
         path     => $path,
         fromrev  => $fromrev,
         torev    => $torev,
