@@ -23,28 +23,27 @@ sub parse_arg {
 
 sub run {
     my ($self, @arg) = @_;
-
     die loc ("Revision required.\n")
 	if $self->{revprop} && !defined $self->{rev};
 
     for my $target (@arg) {
         if ($self->{revprop}) {
-            $self->show_props
+            $self->_show_props
 		( $target,
-		  $target->{repos}->fs->revision_proplist($self->{rev}),
+		  $target->repos->fs->revision_proplist($self->{rev}),
 		  $self->{rev}
 		);
             next;
         }
 
-	$target->as_depotpath ($self->{rev}) if defined $self->{rev};
-        $self->show_props ($target, $self->{xd}->do_proplist ( $target ));
+	$target = $target->as_depotpath ($self->{rev}) if defined $self->{rev};
+        $self->_show_props( $target, $target->root->node_proplist($target->path) );
     }
 
     return;
 }
 
-sub show_props {
+sub _show_props {
     my ($self, $target, $props, $rev) = @_;
 
     %$props or return;
@@ -53,7 +52,7 @@ sub show_props {
         print loc("Unversioned properties on revision %1:\n", $rev);
     }
     else {
-        print loc("Properties on %1:\n", $target->{report} || '.');
+        print loc("Properties on %1:\n", length $target->report ? $target->report : '.');
     }
 
     for my $key (sort keys %$props) {
@@ -70,16 +69,13 @@ sub _arg_revprop {
 sub _proplist {
     my ($self, $target) = @_;
 
-    if ($self->{revprop}) {
-        return $target->{repos}->fs->revision_proplist(
-            (defined($self->{rev}) ? $self->{rev} : $target->{revision})
-        )
-    }
+    return $target->repos->fs->revision_proplist($self->{rev})
+	if $self->{revprop};
 
     if (defined $self->{rev}) {
-        $target->as_depotpath ($self->{rev});
+        $target = $target->as_depotpath ($self->{rev});
     }
-    return $self->{xd}->do_proplist ($target);
+    return $target->root->node_proplist($target->path);
 }
 
 
@@ -99,9 +95,8 @@ SVK::Command::Proplist - List all properties on files or dirs
 
  -R [--recursive]       : descend recursively
  -v [--verbose]         : print extra information
- -r [--revision] arg    : act on revision ARG instead of the head revision
+ -r [--revision] REV    : act on revision REV instead of the head revision
  --revprop              : operate on a revision property (use with -r)
- --direct               : commit directly even if the path is mirrored
 
 =head1 AUTHORS
 
