@@ -5,15 +5,19 @@ use SVK::Version;  our $VERSION = $SVK::VERSION;
 use base qw( SVK::Command );
 use constant opt_recursive => 0;
 use SVK::XD;
-use SVK::Util qw( slurp_fh is_symlink );
+use SVK::Util qw( slurp_fh is_symlink to_native );
 use SVK::I18N;
 
+sub options {
+    ("q|quiet"    => 'quiet');
+}
+
 sub parse_arg {
     my $self = shift;
     my @arg = @_;
     @arg = ('') if $#arg < 0;
 
-    return $self->arg_condensed (@arg);
+    return $self->arg_condensed(@arg);
 }
 
 sub lock {
@@ -22,42 +26,41 @@ sub lock {
 
 sub run {
     my ($self, $target) = @_;
-    my $xdroot = $self->{xd}->xdroot (%$target);
+    my $xdroot = $target->create_xd_root;
 
 	$self->{xd}->checkout_delta
-	    ( %$target,
+	    ( $target->for_checkout_delta,
 	      xdroot => $xdroot,
 	      depth => $self->{recursive} ? undef : 0,
 	      delete_verbose => 1,
 	      absent_verbose => 1,
 	      nodelay => 1,
-	      cb_conflict => \&SVK::Editor::Status::conflict,
-	      cb_unknown => sub {
-		  my ($path, $copath) = @_;
-		  return unless $target->contains_copath ($copath);
-		  print loc("%1 is not versioned; ignored.\n",
-			    $target->report_copath ($copath));
-		  return;
-	      },
+	      cb_conflict => sub { shift->conflict(@_) },
+	      cb_unknown => sub { shift->unknown(@_) },
 	      editor => SVK::Editor::Status->new
 	      ( notify => SVK::Notify->new
 		( cb_flush => sub {
 		      my ($path, $status) = @_;
+		      my $dpath = length $path ? $target->path_anchor."/$path" : $target->path_anchor;
+	              to_native($path);
 		      my $st = $status->[0];
-		      my $dpath = $path ? "$target->{path}/$path" : $target->{path};
 		      my $copath = $target->copath ($path);
 
                       if ($st =~ /[DMRC!]/) {
 			  # conflicted items do not necessarily exist
 			  return $self->do_unschedule ($target, $copath)
-			      if $st eq 'C' && !$xdroot->check_path ($dpath);
+			      if ($st eq 'C' || $status->[2]) && !$xdroot->check_path ($dpath);
                           return $self->do_revert($target, $copath, $dpath, $xdroot);
-                      }
+                      } elsif ($st eq '?') {
+			  return unless $target->contains_copath ($copath);
+			  print loc("%1 is not versioned; ignored.\n",
+			      $target->report_copath ($copath));
+			  return;
+		      }
+
+		      # Check that we are not reverting parents
+		      $target->contains_copath($copath) or return;
 
-                      if ($target->{targets}) {
-                          # Check that we are not reverting parents
-                          $target->contains_copath ($copath) or return;
-                      }
                       $self->do_unschedule($target, $copath);
 		  },
 		),
@@ -72,13 +75,15 @@ sub do_revert {
     # XXX: need to respect copied resources
     my $kind = $xdroot->check_path ($dpath);
     if ($kind == $SVN::Node::dir) {
-	mkdir $copath unless -e $copath;
+        unless (-e $copath) {
+	    mkdir $copath or die loc("Can't create directory while trying to revert %1.\n", $copath);
+        }
     }
     else {
 	# XXX: PerlIO::via::symlink should take care of this.
 	# It doesn't overwrite existing file or close.
 	unlink $copath;
-	my $fh = SVK::XD::get_fh ($xdroot, '>', $dpath, $copath);
+	my $fh = SVK::XD::get_fh ($xdroot, '>', $dpath, $copath) or die loc("Can't create file while trying to revert %1.\n", $copath);
 	my $content = $xdroot->file_contents ($dpath);
 	slurp_fh ($content, $fh);
 	close $fh or die $!;
@@ -91,9 +96,10 @@ sub do_revert {
 
 sub do_unschedule {
     my ($self, $target, $copath) = @_;
-    $self->{xd}{checkout}->store ($copath, { $self->_schedule_empty,
-					     '.conflict' => undef });
-    print loc("Reverted %1\n", $target->report_copath ($copath));
+    $self->{xd}{checkout}->store($copath, { $self->_schedule_empty,
+					    '.conflict' => undef }, {override_descendents => 0});
+    print loc("Reverted %1\n", $target->report_copath ($copath))
+	unless $self->{quiet};
 
 }
 
@@ -112,6 +118,7 @@ SVK::Command::Revert - Revert changes made in checkout copies
 =head1 OPTIONS
 
  -R [--recursive]       : descend recursively
+ -q [--quiet]           : print as little as possible
 
 =head1 AUTHORS
 
