@@ -4,7 +4,7 @@ use strict;
 use SVK::Version;  our $VERSION = $SVK::VERSION;
 
 require SVN::Delta;
-our @ISA = qw(SVN::Delta::Editor);
+use base 'SVK::Editor';
 
 use autouse 'SVK::Util' => qw( tmpfile );
 
@@ -52,7 +52,7 @@ sub callbacks {
     my $self = shift;
     ( cb_exist => sub { $self->cb_exist (@_) },
       cb_localmod => sub { $self->cb_localmod (@_) },
-      cb_localprop => sub { $self->cb_localmod (@_) },
+      cb_localprop => sub { $self->cb_localprop (@_) },
     );
 }
 
