@@ -213,6 +213,7 @@ sub add_file {
 	if (defined $arg[0]) {
 	    warn "===> add merge with history... very bad";
 	}
+	$self->{cb_add_merged}->($path) if $self->{cb_add_merged};
     }
     else {
 	++$self->{changes};
@@ -554,6 +555,7 @@ sub add_directory {
 	    $self->{storage}->open_directory ($path, $self->{storage_baton}{$pdir},
 					      $self->{cb_rev}->($path), $pool);
 	$self->{notify}->node_status ($path, 'G');
+	$self->{cb_add_merged}->($path) if $self->{cb_add_merged};
     }
     else {
 	if (defined $arg[0]) {
@@ -816,10 +818,11 @@ sub delete_entry {
     my ($self, $path, $revision, $pdir, @arg) = @_;
     no warnings 'uninitialized';
     my $pool = $arg[-1];
+    $pool->default;
     my ($basepath, $fromrev) = $self->_resolve_base($path);
     $basepath = $path unless defined $basepath;
 
-    return unless defined $pdir && $self->inspector->exist($basepath, $pool);
+    return unless defined $pdir && $self->inspector->exist($basepath);
     my $rpath = $basepath =~ m{^/} ? $basepath :
 	$self->{base_anchor} eq '/' ? "/$basepath" : "$self->{base_anchor}/$basepath";
     my $torm;
