@@ -1,6 +1,6 @@
 package SVK::Notify;
 use SVK::I18N;
-use SVK::Util qw( abs2rel $SEP to_native get_encoding);
+use SVK::Util qw( abs2rel $SEP to_native from_native get_encoding);
 use strict;
 
 =head1 NAME
@@ -12,6 +12,9 @@ SVK::Notify - svk entry status notification
     $notify = SVK::Notify->new;
     $notify->node_status ('foo/bar', 'M');
     $notify->prop_status ('foo/bar', 'M');
+    $notify->hist_status ('foo/bar', '+',
+	'file://home/foo/.svk/local/trunk/bar', 13);
+    $notify->node_baserev ('foo/bar', 42);
     $notify->flush ('foo/bar');
     $notify->flush_dir ('foo');
 
@@ -40,10 +43,12 @@ sub print_report {
     # XXX: $report should already be in native encoding, so this is wrong
     my $print_native = $enc->name eq 'utf8'
 	? $print
-	: sub { to_native ($_[0]);
+	: sub { to_native($_[0], 'path', $enc);
 		goto \&$print;
 	    };
     return $print_native unless defined $report;
+    $report = "$report";
+    from_native($report, 'path', $enc);
     sub {
 	my $path = shift;
 	if ($target) {
@@ -55,12 +60,12 @@ sub print_report {
 	    }
 	}
 	if (length $path) {
-	    $print_native->($is_copath ? SVK::Target->copath ($report, $path)
+	    $print_native->($is_copath ? SVK::Path::Checkout->copath ($report, $path)
 			               : length $report ? "$report/$path" : $path, @_);
 	}
 	else {
 	    my $r = length $report ? $report : '.';
-	    $print_native->($is_copath ? SVK::Target->copath('', $r) : $r,
+	    $print_native->($is_copath ? SVK::Path::Checkout->copath('', $r) : $r,
 			    @_);
 	}
     };
@@ -95,8 +100,9 @@ sub notify_translate {
 
 sub node_status {
     my ($self, $path, $s) = @_;
+    Carp::cluck unless defined $path;
     $self->{status}{$path}[0] = $s if defined $s;
-    return $self->{status}{$path}[0];
+    return exists $self->{status}{$path} ? $self->{status}{$path}[0] : undef;
 }
 
 my %prop = ( 'U' => 0, 'g' => 1, 'G' => 2, 'M' => 3, 'C' => 4);
@@ -113,17 +119,29 @@ sub prop_status {
 }
 
 sub hist_status {
-    my ($self, $path, $s) = @_;
-    $self->{status}{$path}[2] = $s if defined $s;
+    my ($self, $path, $s, $from_path, $from_rev) = @_;
+    if (defined $s) {
+	$self->{status}{$path}[2] = $s;
+	$self->{copyfrom}{$path} = [$from_path, $from_rev]
+	    if $self->{flush_baserev};
+    }
     return $self->{status}{$path}[2];
 }
 
+sub node_baserev {
+    my ($self, $path, $baserev) = @_;
+    return unless $self->{flush_baserev};
+    $self->{baserev}{$path} = $baserev if defined $baserev;
+}
+
 sub flush {
     my ($self, $path, $anchor) = @_;
     return if $self->{quiet};
     my $status = $self->{status}{$path};
-    if ($status && grep {$_} @{$status}[0..2]) {
-	$self->{cb_flush}->($path, $status) if $self->{cb_flush};
+    if ($status && ($self->{flush_unchanged} || grep {$_} @{$status}[0..1])) {
+	$self->{cb_flush}->($path, $status, $self->{flush_baserev} ?
+		($self->{baserev}{$path}, $self->{copyfrom}{$path}[0], $self->{copyfrom}{$path}[1]) : undef)
+	    if $self->{cb_flush};
     }
     elsif (!$status && !$anchor) {
 	$self->{cb_skip}->($path) if $self->{cb_skip};
