@@ -1,7 +1,6 @@
 package SVK::Target::Universal;
 use strict;
 use SVK::Version;  our $VERSION = $SVK::VERSION;
-use SVK::Util qw(find_svm_source find_local_mirror);
 use base 'Class::Accessor::Fast';
 
 __PACKAGE__->mk_accessors(qw(uuid path rev));
@@ -12,37 +11,24 @@ SVK::Target::Universal - svk target that might not be local
 
 =head1 SYNOPSIS
 
- $target = SVK::Target::Universal->new ($uuid, $path, $rev);
- $target = SVK::Target::Universal->new ($local_target);
- $local_target = $target->local ($xd, 'depot');
+ $target = SVK::Target::Universal->new($uuid, $path, $rev);
+ $local_target = $target->local($depot);
 
 =cut
 
 sub new {
     my $class = shift;
-    return $class->new (find_svm_source ($_[0]->repos, $_[0]->path, $_[0]->revision))
-	if ref ($_[0]);
-
     my ($uuid, $path, $rev) = @_;
-    bless { uuid => $uuid,
-	    path => $path,
-	    rev => $rev }, $class;
+    $class->SUPER::new( { uuid => $uuid,
+                          path => $path,
+                          rev  => $rev } );
 }
 
 sub local {
-    my $self = shift;
-    my ($repospath, $repos, $depot, $xd);
-    if ($#_) {
-	($xd, $depot) = @_;
-	($repospath, undef, $repos) = $xd->find_repos ("/$depot/", 1);
-    }
-    else {
-	$repos = $_[0];
-	$repospath = $repos->path;
-    }
+    my ($self, $depot) = @_;
 
-    my ($path, $rev) = $self->{uuid} ne $repos->fs->get_uuid ?
-	find_local_mirror ($repos, @{$self}{qw/uuid path rev/}) :
+    my ($path, $rev) = $self->{uuid} ne $depot->repos->fs->get_uuid ?
+	$depot->find_local_mirror(@{$self}{qw/uuid path rev/}) :
 	@{$self}{qw/path rev/};
 
     # $rev can be undefined even if $path is defined.  This is the case
@@ -51,12 +37,10 @@ sub local {
     return unless defined $path && defined $rev;
 
     SVK::Path->real_new
-	({ repos => $repos,
-	   mirror => $xd ? $xd->mirror($repos) : undef,
-	   repospath => $repospath,
+	({ depot => $depot,
+	   mirror => $depot->mirror,
 	   path => $path, # XXX: use path_anchor accessor
 	   revision => $rev,
-	   depotname => $depot || '',
 	 });
 }
 
