@@ -9,7 +9,7 @@ our @EXPORT_OK = qw(
 
     get_encoding get_encoder from_native to_native
 
-    find_local_mirror find_svm_source resolve_svm_source traverse_history
+    find_svm_source traverse_history
     find_prev_copy
 
     read_file write_file slurp_fh md5_fh bsd_glob mimetype mimetype_is_text
@@ -327,74 +327,13 @@ sub to_native {
     return;
 }
 
-=head2 Mirror Handling
-
-=head3 find_local_mirror ($repos, $uuid, $path, $rev)
-
-XXX Undocumented
-
-=cut
-
-sub find_local_mirror {
-    my ($repos, $uuid, $path, $rev) = @_;
-    my $myuuid = $repos->fs->get_uuid;
-    return unless HAS_SVN_MIRROR && $uuid ne $myuuid;
-    my ($m, $mpath) = _has_local ($repos, "$uuid:$path");
-    return ("$m->{target_path}$mpath",
-	    $rev ? $m->find_local_rev ($rev) : $rev) if $m;
-}
-
-=head3 find_svm_source ($repos, $path, $rev)
-
-XXX Undocumented
-
-=cut
-
-sub find_svm_source {
+sub find_svm_source { # DEPRECATED: use SVK::Path->universal, only used in SVK::Command now.
     my ($repos, $path, $rev) = @_;
-    my $fs = $repos->fs;
-    $rev ||= $fs->youngest_rev;
-    my $root = $fs->revision_root ($rev);
-    my ($uuid, $m, $mpath);
-
-    if (HAS_SVN_MIRROR) {
-	($m, $mpath) = SVN::Mirror::is_mirrored ($repos, $path);
-    }
-
-    if ($m) {
-	# XXX: we should normalize $rev before calling find_svm_source
-	$rev = ($root->node_history($path)->prev(0)->location)[1]
-	    unless $rev == $root->node_created_rev ($path);
-	$rev = $m->find_remote_rev ($rev);
-	$path =~ s/\Q$mpath\E$//;
-	$uuid = $m->{source_uuid};
-	$path = $m->{source_path}.$mpath;
-	$path ||= '/';
-    }
-    else {
-	$uuid = $fs->get_uuid;
-	$rev = ($root->node_history ($path)->prev (0)->location)[1];
-    }
-
-    return ($uuid, $path, $rev);
-}
-
-=head3 resolve_svm_source ($repos, $uuid, $path)
-
-XXX Undocumented
-
-=cut
-
-# XXX: deprecated, only used in command::verify now. use
-# $target->is_mirrored instead
-sub resolve_svm_source {
-    my ($repos, $uuid, $path) = @_;
-    my $myuuid = $repos->fs->get_uuid;
-    return ($path) if ($uuid eq $myuuid);
-    return unless HAS_SVN_MIRROR;
-    my ($m, $mpath) = _has_local ($repos, "$uuid:$path");
-    return unless $m;
-    return ("$m->{target_path}$mpath", $m);
+    my $t = SVK::Path->real_new({ depot => SVK::Depot->new({repos => $repos}),
+                                  path => $path, revision => $rev });
+    $t->refresh_revision unless $rev;
+    my $u = $t->universal;
+    return map { $u->$_ } qw(uuid path rev);
 }
 
 =head2 File Content Manipulation
@@ -1036,24 +975,6 @@ sub find_dotsvk {
     return
 }
 
-sub _list_mirror_cached {
-    my $repos = shift;
-    SVK::Mirror->new({repos => $repos})->entries;
-}
-
-sub _has_local {
-    my ($repos, $spec) = @_;
-    my %mirrored = _list_mirror_cached($repos);
-    while (my ($path, $m) = each(%mirrored)) {
-	my $mspec = $m->spec;
-	my $mpath = $spec;
-	next unless $mpath =~ s/^\Q$mspec\E//;
-	$mpath = '' if $mpath eq '/';
-	return ($m->mirror, $mpath);
-    }
-    return;
-}
-
 1;
 
 __END__
